"""
Python Wrapper for Govtrack API v2

Based off of https://github.com/cdelguercio/govtrack-python and https://github.com/markgx/govtrack-node/

"""

import json
import requests
import urllib

API_BASE = "https://www.govtrack.us/api/v2/"
VALID_ENDPOINTS = ["bill", "cosponsership", "person", "role", "vote", "vote_voter", "committee", "committe_member"]
class GTPY(object):
    
    def _param_encode(self, params):
        """Checks if the parameters is just the id, if it is, pass it down. Otherwise pop out the id and parse accordingly"""
        _id = None
        if type(params) == int:
            _id = params
            return _id
        else:
            params = {k: self._jsonify(v) for k, v in params.items()}
            _id = params.get("id")
            if _id:
                _params = params.pop("id")
        _params = "?" + urllib.parse.urlencode(params) if any(params) == True else ""
        _params = _id + _params if _id else _params
        return _params

    def _call(self, params):
        """docstring for _call"""
        url = API_BASE + params
        print(url)
        response = requests.get(url)
        return response.json()
    
    def get(self, endpoint, params, obj_only):
        """
        calls the passed endpoint, returns a JSON object or a list if obj_only is true
        """
        endpoint += "/"
        data = self._call(endpoint + str(self._param_encode(params)))
        return data["objects"] if obj_only else data
        
    def _jsonify(self, val):
        """Transforms python True/False into json strings, there's probably a fucntion for this already built somewhere"""
        if val == True:
            return "true"
        elif val == False:
            return "false"
        else:
            return val

    def bill(self, params, obj_only = False):
        return self.get("bill", params, obj_only)
        
    def cosponsorship(self, params, obj_only = False):
        return self.get("cosponsorship", params, obj_only)
    
    def person(self, params, obj_only = False):
        return self.get("person", params, obj_only)
    
    def role(self, params, obj_only = False):
        return self.get("role", params, obj_only)
    
    def vote(self, params, obj_only = False):
        return self.get("vote", params, obj_only)
        
    def vote_voter(self, params, obj_only = False):
        return self.get("vote_voter", params, obj_only)
        
    def committee(self, params):
        return self.get("committee", params, False)
    
    def committee_member(self, params, obj_only = False):
        return self.get("committee_member", params, obj_only)





from unittest2 import TestCase

from api import GTPY

class GTPYTests(TestCase):
    def setUp(self):
        self.client = GTPY()
    
    def test_url_encode(self):
        #encoding a regular dictionary w/o an id
        test_params = {'lastname': 'Kennedy'}
        encode_test = self.client._param_encode(test_params)
        self.assertEqual(encode_test, "?lastname=Kennedy")

        #encoding an id
        test_params = {"id": "40054"}
        encode_test = self.client._param_encode(test_params)
        self.assertEqual(encode_test, "40054")

        #encoding an id and other values
        test_params = {"id": "40054", "lastname": "Kennedy"}
        encode_test = self.client._param_encode(test_params)
        self.assertEqual(encode_test, "40054?lastname=Kennedy")
        
    def test_role(self):
        """Tests for the role"""
        #tests the status with python bools and role limits
        TEST_ROLE_STATUS = True
        TEST_ROLE_LIMIT = 50
        test_params = {"current": TEST_ROLE_STATUS, "limit": TEST_ROLE_LIMIT}
        response = self.client.role(test_params)
        self.assertEqual(response["meta"]["limit"], TEST_ROLE_LIMIT)
        self.assertEqual(response["objects"][0]["current"], TEST_ROLE_STATUS)
        
        response = self.client.role(test_params, True)
        self.assertEqual(response[0]["current"], TEST_ROLE_STATUS)
        
    def test_person(self):
        """Tests for person"""
        
        TEST_ID = 400054
        response = self.client.person(TEST_ID)
        self.assertTrue("id" in response and response["id"] == TEST_ID) 
        self.assertTrue("name" in response and "roles" in response)
        
    def test_cosponsorship(self):
        """Tests for the cosponsorship"""
        
        TEST_BILL_ID = 345256
        response = self.client.cosponsorship({"bill": TEST_BILL_ID})
        self.assertEqual(response["objects"][0]["bill"], TEST_BILL_ID)
        
        response = self.client.cosponsorship({"bill": TEST_BILL_ID}, True)
        self.assertEqual(response[0]["bill"], TEST_BILL_ID)
        
        TEST_BILL_ID = 1
        response = self.client.cosponsorship(TEST_BILL_ID)
        self.assertEqual(response["id"], TEST_BILL_ID)
        
    def test_vote(self):
        """Tests the votes by calling vote ID 1"""
        
        TEST_VOTE_ID = 1
        response = self.client.vote(TEST_VOTE_ID)
        self.assertEqual(response["id"], TEST_VOTE_ID)
        
        TEST_VOTE_CONGRESS = 112
        TEST_VOTE_CHAMBER = "house"
        TEST_VOTE_SESSION = 2011
        response = self.client.vote({"congress": TEST_VOTE_CONGRESS, "chamber": TEST_VOTE_CHAMBER, "session": TEST_VOTE_SESSION})
        self.assertEqual(response["objects"][0]["id"], 1015)
        
        response = self.client.vote({"congress": TEST_VOTE_CONGRESS, "chamber": TEST_VOTE_CHAMBER, "session": TEST_VOTE_SESSION}, True)
        self.assertEqual(response[0]["id"], 1015)
        
    
    def test_vote_voter(self):
        """Tests by finding Mark Kirk's voting record"""
        TEST_VOTER_PERSON_ID = 400222
        
        response = self.client.vote_voter({"person": TEST_VOTER_PERSON_ID}, True)
        self.assertEqual(response[0]["person"]["firstname"], "Mark")
    
    def test_committee(self):
        """Tests by searching for the Senate Judicaiary comittee"""
        TEST_COMMITTEE_ID = 2650
        
        response = self.client.committee(TEST_COMMITTEE_ID)
        self.assertEqual(response["abbrev"], "Judiciary")
        self.assertEqual(response["committee_type"], "senate")
    
    def test_committee_memember(self):
        """Tests by getting Ralph Abraham, then committee 2703"""
        TEST_COMMITTEE_MEMBER_ID = 204199
        
        response = self.client.committee_member(TEST_COMMITTEE_MEMBER_ID)
        self.assertEqual(response["person"]["firstname"], "Ralph")
        
        TEST_COMMITTEE_ID = "2703"
        
        response = self.client.committee_member({"committee": TEST_COMMITTEE_ID}, True)
        self.assertEqual(response[0]["committee"]["code"], "HSIF")


from setuptools import setup

def readme():
    with open('readme.md') as f:
        return f.read()

setup(
    name='gt-py',
    packages=['gtpy'],
    install_requires=['unittest2==1.1.0', 'requests==2.9.1'],
    version='1.2',
    license='MIT',
    description='Python library for GovTrack.us API',
    author='Peixian Wang',
    author_email='peixian@driftingicebergs.com',
    url='https://github.com/peixian/gt-py',
    download_url='https://github.com/peixian/gt-py/archive/1.2.tar.gz',
    keywords=['api', 'python', 'govtrack', 'gt'],
    classifiers=[],
    test_suite='nose.collector',
    tests_require=['nose', 'nose-cover3'],
)
